"""BioMASS - A Python Framework for Modeling and Analysis of Signaling Systems"""

from .construction import Text2Model
from .core import *
from .result import OptimizationResults
from .version import __version__

__author__ = __maintainer__ = "Hiroaki Imoto"
__email__ = "hiroaki.imoto@ucd.ie"
